import { Provider, forwardRef, Type, ForwardRefFn } from '@angular/core';
import { ControlValueAccessor, NG_VALUE_ACCESSOR } from '@angular/forms';

export abstract class AbstractValueAccessor implements ControlValueAccessor {
    protected _value: any = null;
    protected _previousValue: any = null;
    public onChange = (_) => { };
    public onTouched = () => { };

    public get value(): any {
      return this._value;
    }

    public set value(v: any) {
      if (v !== this._value) {
        this._previousValue = this._value;
        this._value = v;
        this.onChange(v);
      }
    }

    public writeValue(value: any) {
      this._value = value;
      
      
      
      //IMPORTANT NOTE: ngModelChange should only be fired when the component changes the value of the model
      //not when the component receives a value from a model
      //this.onChange(value); ---->>>>>> this fired ngModelChange event
    }

    public registerOnChange(fn: (_: any) => void): void {
      this.onChange = fn;
    }

    public registerOnTouched(fn: () => void): void {
      this.onTouched = fn;
    }
}

// export function MakeProvider(forwardRefFn: ForwardRefFn) {
//   return {
//     provide: NG_VALUE_ACCESSOR,
//     useExisting: forwardRef(forwardRefFn),
//     multi: true
//   };
// }