import { Component, Input, EventEmitter, Output, forwardRef } from '@angular/core';
import { ControlValueAccessor, NG_VALUE_ACCESSOR } from '@angular/forms';
import { TemplateParserComponent } from '../../baseclasses/template-parser.component-base';


@Component({
  selector: 'cbx-checkbox',
  templateUrl: './cbx-checkbox.component.html',
  styleUrls: ['./cbx-checkbox.component.less'],
  host:{
    '[tabindex]':'tabindex',
    '[class.cbx]':'true',
    '[class.cbx-state-checked]':'checked',
    '[class.cbx-state-disabled]':'disabled',
    '(click)':'onTogglerClick($event)',
    '(keyup.enter)':'onTogglerClick($event)'
  },
  providers:[
    {
      provide: NG_VALUE_ACCESSOR,
      useExisting: forwardRef(() => CbxCheckbox),
      multi: true
    }
  ]
})
export class CbxCheckbox extends TemplateParserComponent implements ControlValueAccessor {
  
  @Input() tabindex = "-1";
  @Input() label:string = "";
  @Input() disabled:boolean = false;
  @Input() required: boolean = false;
  @Input() strTooltip:string = "";

  //ControlValueAccessor | start
  public onChange = (_) => { };
  public onTouched = () => { };

  writeValue(obj: any): void {
    this.checked = !!obj;//double negation to ensure value is a bool
  }

  registerOnChange(fn: any): void {
    this.onChange = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouched = fn;
  }
  //ControlValueAccessor | end

  
  checked:boolean = null;

  @Input('checked') set _checked(val:any){
    this.checked = !!val;//double negation to ensure boolean value;
    this.onChange(this.checked);
  }
  
  @Output() cbxChange:EventEmitter<boolean> = new EventEmitter();

  onTogglerClick(event:Event){
    if(this.disabled){
      return;
    }

    this.checked = !this.checked;
    this.onChange(this.checked);
    this.onTouched();
    this.cbxChange.emit(this.checked)
  }
}
