import { Component, OnInit, forwardRef, EventEmitter, Input, Output, ViewChild, SimpleChanges } from '@angular/core';
import { DatePipe } from '@angular/common';

import { NG_VALUE_ACCESSOR, ControlValueAccessor } from '@angular/forms';
import { startOfQuarter, endOfQuarter, createDate, endOfYear, startOfYear, endOfLastWeek, startOfLastWeek, startOfLastMonth, 
    endOfLastMonth, startOfLastQuarter, endOfLastQuarter, startOfLastYear, endOfLastYear, cloneDate } from '../cbx-input-date/utilities/date-utils';
import { CbxInputDate } from '../cbx-input-date/cbx-input-date.component';
import { cbxToolTipVisbilityConfig } from '../../directives/cbx-tooltip.directive';
export class DateRange {
    startDate: Date;
    endDate: Date;
    constructor(){}
}

export class SaveDateState {
    date:Date = null;
    startDate:Date = null;
    endDate:Date = null;
    monthDays:any[] = [];
    toMonthDays:any[] = [];
    preConfiguredRange:number = -1;
}

export interface DatePickerSettings{
    bigBanner?: Boolean;
    timePicker: Boolean;
    timePickerOnly?: Boolean;
    format: string;
    formatPlaceholder:string;
    placeholder: string;
    defaultOpen?: Boolean;
    cal_days_labels: Array<string>;
    cal_full_days_lables: Array<string>;
    cal_months_labels: Array<string>;
    cal_months_labels_short: Array<string>;
    closeOnSelect?: boolean;
    rangepicker?: boolean;

}

export const DATEPICKER_CONTROL_VALUE_ACCESSOR: any = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => CbxDatePicker),
    multi: true
};

@Component({
    selector: 'cbx-datepicker',
    templateUrl: './datepicker.component.html',
    styleUrls: ['./datepicker.component.less', './rangepicker.less'],
    providers: [DATEPICKER_CONTROL_VALUE_ACCESSOR]
})

export class CbxDatePicker  implements OnInit, ControlValueAccessor {

    @ViewChild('rangeStartDateInput') rangeStartDateInput: CbxInputDate;
    @ViewChild('rangeEndDateInput') rangeEndDateInput: CbxInputDate;
    @ViewChild('dateInput') dateInput: CbxInputDate;
    @ViewChild('timeInput') timeInput: CbxInputDate;

    @Input()label:string;
    @Input() strTooltip:string = null;
    @Input()preConfiguredRange: number;
    @Input()settings: DatePickerSettings;
    @Input() pickerType:string = null;
    @Input() required: boolean = false;

    @Output()onDateSelect: EventEmitter<Date> = new EventEmitter<Date>();

    datePickerTooltipConfig:cbxToolTipVisbilityConfig = {
        showOnClick: true,
        hideOnClickout: true
    };

    selectedDate: String;
    _date: Date;
    get date(){
        return this._date;
    }
    set date(value){
        this._date = value;
        if (value) {
            this._hour = value.getHours();
            this._minute = value.getMinutes();
        }
    }   

    _hour: number;
    get hour(){
        return this._hour;
    }
    set hour(value){
        this._hour = value;
        var newDate = new Date(this._date);
        newDate.setHours(value);
        this.date = newDate;
        this.onChangeCallback(this.date.toString());
    }

    _minute: number;
    get minute(){
        return this._minute;
    }
    set minute(value){
        this._minute = value;
        var newDate = new Date(this._date);
        newDate.setMinutes(value);
        this.date = newDate;
        this.onChangeCallback(this.date.toString());
    }
    dateRange: DateRange = new DateRange();
    _popover: boolean;
    get popover() {
        return this._popover;
    }
    set popover(value) {
        this._popover = value;
        if (value) {
            if (this.date == null)  {
                this.date = new Date();
            }
            if (!this.settings.rangepicker ) {
                if (this.monthDays == null || this.monthDays.length == 0) {
                    this.initDate(this.date.toString());
                    this.monthDays = this.generateDays(this.date);
                }
            } else {
                if (this.monthDays == null || this.monthDays.length == 0 || this.toMonthDays == null || this.toMonthDays.length == 0) {
                    var val = {
                        startDate: new Date(),
                        endDate: new Date(),
                    }
                    if (this.dateRange.startDate != null) {
                        val.startDate = this.dateRange.startDate;
                    }
                    if (this.dateRange.endDate != null) {
                        val.endDate = this.dateRange.endDate;
                    }

                    this.initDateRange(val);
                    this.initMonthDaysForRange();
                }
            }
            this.savedDateState = {
                date: this.date,
                startDate: this.dateRange.startDate,
                endDate: this.dateRange.endDate,
                monthDays: this.monthDays,
                toMonthDays: this.toMonthDays,
                preConfiguredRange: this.preConfiguredRange
            }
        }
    }


    cal_days_in_month: Array<any> = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
    timeViewDate: Date = new Date(this.date);
    hourValue: number = 0;
    toHourValue: number = 0;
    minValue: number = 0;
    toMinValue: number = 0;
    timeViewMeridian: string = "";
    toTimeViewMeridian: string = "";
    timeView: boolean = false;
    yearView: Boolean = false;
    yearsList: Array<any> = [];
    monthDays: Array<any> = [];
    toMonthDays: Array<any> = [];
    monthsView: boolean = false;
    today: Date = new Date();
    leftDate: Date = new Date();
    rightDate: Date = new Date();
    savedDate: Date;
    hasHover: boolean = false;  
    defaultSettings: DatePickerSettings = {
        defaultOpen: false,
        bigBanner: true,
        timePicker: true,
        timePickerOnly: false,
        format: 'MM/dd/yyyy hh:mm a',
        formatPlaceholder: 'MM-dd-yyyy hh:mm a',
        placeholder: ' ',
        cal_days_labels: ['Su', 'Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa'],
        cal_full_days_lables: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
        cal_months_labels: ['January', 'February', 'March', 'April',
            'May', 'June', 'July', 'August', 'September',
            'October', 'November', 'December'],
        cal_months_labels_short: ['JAN', 'FEB', 'MAR', 'APR',
            'MAY', 'JUN', 'JUL', 'AUG', 'SEP',
            'OCT', 'NOV', 'DEC'],
        closeOnSelect: true,
        rangepicker: false,
    }

    savedDateState: SaveDateState = {
        date: null,
        startDate: null,
        endDate: null,
        monthDays: [],
        toMonthDays: [],
        preConfiguredRange: -1
    }

    todayDate:Date = null;

    constructor() {
        this.preConfiguredRange = -1;  
        this.todayDate = new Date();
    }
    ngOnInit() {
        this.settings = Object.assign(this.defaultSettings, this.settings);
        if (this.settings.defaultOpen) {
            this.popover = true;
        } 
    }

    ngOnChanges(changes:SimpleChanges){
        if(changes.pickerType){
            this.setDefaultSettings();
        }
    }

    /** set default settings base on picker type */
    setDefaultSettings() {
        if (this.pickerType == "date") {
            let customSettings = {
                bigBanner: true,
                timePicker: false,
                format: "MM/dd/yyyy",
                defaultOpen: false,
                closeOnSelect: false,
                rangepicker: false
            };

            this.settings = {
                ...this.defaultSettings,
                ...customSettings
            };

        } else if (this.pickerType == "datetime") {
            let customSettings = {
                bigBanner: true,
                timePicker: true,
                defaultOpen: false,
                closeOnSelect: false
            };

            this.settings = {
                ...this.defaultSettings,
                ...customSettings
            };

        } else if (this.pickerType == "time") {
            let customSettings = {
                bigBanner: false,
                timePicker: true,
                timePickerOnly: true,
                format: "hh:mm a",
                defaultOpen: false,
                closeOnSelect: false,
            };

            this.settings = {
                ...this.defaultSettings,
                ...customSettings
            };
        }
    }

    private onTouchedCallback = () => {
        
    };
    
    public onChangeCallback = (_: any) => {
        
    };

    writeValue(value: any) {
        if (value !== undefined && value !== null) {
            if (!this.settings.rangepicker) {
                this.initDate(value);
                this.monthDays = this.generateDays(this.date);
            }
            else {
                this.initDateRange(value);
                this.initMonthDaysForRange();
             }
        }
        else {
            this.date = null;
        }
    }



    registerOnChange(fn: any) {
        this.onChangeCallback = fn;
    }

    registerOnTouched(fn: any) {
        this.onTouchedCallback = fn;
    }
    initDate(val: string) {
        this.date = new Date(val);
        if (this.date.getHours() <= 11) {
            this.hourValue = this.date.getHours();
            this.timeViewMeridian = "AM";
        }
        else {
            this.hourValue = this.date.getHours() - 12;
            this.timeViewMeridian = "PM";
        }
        if (this.date.getHours() == 0 || this.date.getHours() == 12) {
            this.hourValue = 12;
        }
        this.minValue = this.date.getMinutes();
    }
    initDateRange(val: DateRange) {
        if (val.startDate) {
            this.dateRange.startDate = new Date(val.startDate);
            if (this.dateRange.startDate.getHours() <= 11) {
                this.hourValue = this.dateRange.startDate.getHours();
                this.timeViewMeridian = "AM";
            }
            else {
                this.hourValue = this.dateRange.startDate.getHours() - 12;
                this.timeViewMeridian = "PM";
            }
            if (this.dateRange.startDate.getHours() == 0 || this.dateRange.startDate.getHours() == 12) {
                this.hourValue = 12;
            }
            this.minValue = this.dateRange.startDate.getMinutes();
        } else {
            this.dateRange.startDate = null
        }

        if (val.endDate) {
            this.dateRange.endDate = new Date(val.endDate);
            if (this.dateRange.endDate.getHours() <= 11) {
                this.toHourValue = this.dateRange.endDate.getHours();
                this.toTimeViewMeridian = "AM";
            }
            else {
                this.toHourValue = this.dateRange.endDate.getHours() - 12;
                this.toTimeViewMeridian = "PM";
            }
            if (this.dateRange.endDate.getHours() == 0 || this.dateRange.endDate.getHours() == 12) {
                this.toHourValue = 12;
            }
            this.toMinValue = this.dateRange.endDate.getMinutes();
        } else {
            this.dateRange.endDate = null;
        }





    }

    initMonthDaysForRange() {
        if (this.dateRange.startDate != null && this.dateRange.endDate != null && this.dateRange.startDate.getMonth() === this.dateRange.endDate.getMonth() && this.dateRange.startDate.getFullYear() === this.dateRange.endDate.getFullYear()) {
            this.leftDate = new Date(this.dateRange.startDate);
            var tempDate = new Date(this.dateRange.startDate);
            tempDate.setMonth(tempDate.getMonth() + 1);
            tempDate.setDate(1);
            this.rightDate = new Date(tempDate);
            this.monthDays = this.generateDays(this.leftDate);
            this.toMonthDays = this.generateDays(this.rightDate);
        }
        else {
            this.leftDate = new Date(this.dateRange.startDate);
            this.rightDate = new Date(this.dateRange.endDate);
            this.monthDays = this.generateDays(this.leftDate);
            this.toMonthDays = this.generateDays(this.rightDate);
        }
    }
    generateDays(date: Date) {
        var year = date.getFullYear(),
            month = date.getMonth(),
            current_day = date.getDate(),
            today = new Date();
        var firstDay = new Date(year, month, 1);
        var startingDay = firstDay.getDay();
        var monthLength = this.getMonthLength(month, year);
        var day = 1;
        var dateArr = [];
        var dateRow = [];
        // this loop is for is weeks (rows)
        for (var i = 0; i < 9; i++) {
            // this loop is for weekdays (cells)
            dateRow = [];
            for (var j = 0; j <= 6; j++) {
                var dateCell = null;
                if (day <= monthLength && (i > 0 || j >= startingDay)) {
                    dateCell = day;
                    if (day == current_day) {
                        // dateCell.classList.add('selected-day');
                    }
                    if (day == today.getDate() && date.getMonth() == today.getMonth() && date.getFullYear() == today.getFullYear()) {
                        // dateCell.classList.add('today');
                    }
                    day++;
                }
                dateRow.push({ day: dateCell, date: new Date((month + 1) + '-' + dateCell + '-' + date.getFullYear()) });
            }
            // stop making rows if we've run out of days
            if (day > monthLength) {
                dateArr.push(dateRow);
                break;
            } else {
                dateArr.push(dateRow);
            }
        }
        var currentYear = (this.date||this.todayDate).getFullYear();
        var startYear = currentYear - 4;
        for (var k = 0; k < 9; k++) {
            this.yearsList[k] = startYear + k;
        }
        return dateArr;
    }
    generateYearList(param: string) {
        var startYear = null;
        var currentYear = null;
        if (param == "next") {
            startYear = this.yearsList[8] + 1;
            currentYear = this.date.getFullYear();
        }
        else if (param == "prev") {
            startYear = this.yearsList[0] - 9;
            currentYear = this.date.getFullYear();
        }
        else {
            currentYear = this.date.getFullYear();
            startYear = currentYear - 4;
            this.yearView = !this.yearView;
            this.monthsView = false;
        }
        for (var k = 0; k < 9; k++) {
            this.yearsList[k] = startYear + k;
        }
    }
    getMonthLength(month: number, year: number) {
        var monthLength = this.cal_days_in_month[month];

        // compensate for leap year
        if (month == 1) { // February only!
            if ((year % 4 == 0 && year % 100 != 0) || year % 400 == 0) {
                monthLength = 29;
            }
        }
        return monthLength;
    }
    toggleMonthView() {
        this.yearView = false;
        this.monthsView = !this.monthsView;
    }
    toggleYearView() {
        this.monthsView = false;
        this.yearView = !this.yearView;
    }
    toggleMeridian(val: string) {
        this.timeViewMeridian = val;
    }
    setTimeView() {
        if (this.timeViewMeridian == "AM") {
            if (this.hourValue == 12) {
                this.date.setHours(0);
            }
            else {
                this.date.setHours(this.hourValue);
            }
            this.date.setMinutes(this.minValue);
        }
        else {
            if (this.hourValue == 12) {
                this.date.setHours(this.hourValue);
            }
            else {
                this.date.setHours(this.hourValue + 12);
            }
            this.date.setMinutes(this.minValue);
        }
        this.date = new Date(this.date);
        this.timeView = !this.timeView;
    }

    /***
     * (ssd > endDay -> startDay = endDay -> step = 1 ) && (sed > startDay -> 2)
     * (ssd < endDay -> startDay = ssd - step =1) && (sed < startDay -> 2 )
     * 
     */

    rangeSelected: number = 0;
    setDay(evt: any, type?: string) {
        if (evt.target.innerHTML) {
            var selectedDay = new Date(evt.target.getAttribute('data-label'));
            if (type == 'range') {
                if (this.rangeSelected == 0) {
                    this.setStartDate(selectedDay);
                }
                else if (this.rangeSelected == 1) {
                    this.setEndDate(selectedDay);
                }
                this.preConfiguredRange = -1;
            }
            else {
                var prevDate = this.date;
                let newDate = new Date(selectedDay);
                newDate.setHours(prevDate.getHours());
                newDate.setMinutes(prevDate.getMinutes());
                newDate.setSeconds(prevDate.getSeconds());
                newDate.setMilliseconds(prevDate.getMilliseconds());

                this.date = newDate;
                this.onChangeCallback(this.date.toString());
            }
            if (this.settings.closeOnSelect) {
                this.popover = false;
                this.onDateSelect.emit(this.date);
            }
        }
    }
    setStartDate(selectedDate: Date) {
        if (selectedDate < this.dateRange.endDate) {
            this.dateRange.startDate = new Date(selectedDate);
        }
        else if (selectedDate > this.dateRange.endDate) {
            this.dateRange.startDate = new Date(selectedDate);
            this.dateRange.endDate = new Date(selectedDate);
        }
        this.rangeSelected = 1;
    }
    setEndDate(selectedDate: Date) {
        if (selectedDate > this.dateRange.startDate && (this.dateRange.startDate != this.dateRange.endDate)) {
            this.dateRange.endDate = new Date(selectedDate);
        }
        else if (selectedDate > this.dateRange.startDate && (this.dateRange.startDate == this.dateRange.endDate)) {
            this.dateRange.endDate = new Date(selectedDate);
        }
        else if (selectedDate < this.dateRange.startDate && (this.dateRange.startDate != this.dateRange.endDate)) {
            this.dateRange.startDate = new Date(selectedDate);
            this.dateRange.endDate = new Date(selectedDate);
        }
        else if (selectedDate < this.dateRange.startDate && (this.dateRange.startDate == this.dateRange.endDate)) {
            this.dateRange.startDate = new Date(selectedDate);
            this.dateRange.endDate = new Date(selectedDate);
        }
        else if (selectedDate.getTime() == this.dateRange.startDate.getTime()) {
            this.dateRange.startDate = new Date(selectedDate);
            this.dateRange.endDate = new Date(selectedDate);
        }
        this.rangeSelected = 0;
    }
    highlightRange(date: Date) {
        return (date > this.dateRange.startDate && date < this.dateRange.endDate);

    }
    setYear(evt: any) {
        console.log(evt.target);
        var selectedYear = parseInt(evt.target.getAttribute('id'));
        this.date = new Date(this.date.setFullYear(selectedYear));
        this.yearView = !this.yearView;
        this.monthDays = this.generateDays(this.date);
    }
    setMonth(evt: any) {
        if (evt.target.getAttribute('id')) {
            var selectedMonth = this.settings.cal_months_labels_short.indexOf(evt.target.getAttribute('id'));
            this.date = new Date(this.date.setMonth(selectedMonth));
            this.monthsView = !this.monthsView;
            this.monthDays = this.generateDays(this.date);
        }
    }
    prevMonth(e: any) {
        e.stopPropagation();
        var self = this;
        if (this.date.getMonth() == 0) {
            this.date.setMonth(11);
            this.date.setFullYear(this.date.getFullYear() - 1);
        } else {
            var prevmonthLength = this.getMonthLength(this.date.getMonth() - 1, this.date.getFullYear());
            var currentDate = this.date.getDate();
            if (currentDate > prevmonthLength) {
                this.date.setDate(prevmonthLength);
            }
            this.date.setMonth(this.date.getMonth() - 1);
        }
        this.date = new Date(this.date);
        this.monthDays = this.generateDays(this.date);
    }
    nextMonth(e: any) {
        e.stopPropagation();
        var self = this;
        if (this.date.getMonth() == 11) {
            this.date.setMonth(0);
            this.date.setFullYear(this.date.getFullYear() + 1);
        } else {
            var nextmonthLength = this.getMonthLength(this.date.getMonth() + 1, this.date.getFullYear());
            var currentDate = this.date.getDate();
            if (currentDate > nextmonthLength) {
                this.date.setDate(nextmonthLength);
            }
            this.date.setMonth(this.date.getMonth() + 1);

        }
        this.date = new Date(this.date);
        this.monthDays = this.generateDays(this.date);
    }
    onChange(evt: any) {
        console.log(evt);
    }
    incHour() {
        if (this.hourValue < 12) {
            this.hourValue += 1;
            console.log(this.hourValue);
        }
    }
    decHour() {
        if (this.hourValue > 1) {
            this.hourValue -= 1;
            console.log(this.hourValue);
        }
    }
    incMinutes() {
        if (this.minValue < 59) {
            this.minValue += 1;
            console.log(this.minValue);
        }
    }
    decMinutes() {
        if (this.minValue > 0) {
            this.minValue -= 1;
            console.log(this.minValue);
        }
    }
    done() {
        this.onChangeCallback(this.date.toString());
        this.popover = false;
        this.onDateSelect.emit(this.date);
    }
    togglePopover() {
        if (this.popover) {
            this.closepopover();
        }
        else {
            this.popover = true;
        }

    }
    closepopover() {
        this.rangeSelected = 0;
        this.popover = false;
    }
    composeDate(date: Date) {
        return (date.getMonth() + 1) + '-' + date.getDate() + '-' + date.getFullYear();
    }
    getCurrentWeek() {
        var curr_date = new Date();

        var day = curr_date.getDay();

        var diff = curr_date.getDate() - day + (day == 0 ? -6 : 1); // 0 for sunday

        var week_start_tstmp = curr_date.setDate(diff);

        var week_start = new Date(week_start_tstmp);


        var week_end = new Date(week_start_tstmp);  // first day of week 

        week_end = new Date(week_end.setDate(week_end.getDate() + 6));


        var date = week_start + ' to ' + week_end;    // date range for current week
        console.log(date);
        if (week_start.getMonth() === week_end.getMonth()) {
            this.monthDays = this.generateDays(week_start);
            var tempDate = new Date(week_end);
            tempDate.setMonth(tempDate.getMonth() + 1);
            tempDate.setDate(1);
            this.toMonthDays = this.generateDays(tempDate);
        }
        else {
            this.monthDays = this.generateDays(week_start);
            this.toMonthDays = this.generateDays(week_end);
        }

        this.setStartDate(week_start);
        this.setEndDate(week_end);
    }


    saveDate(){
        this.savedDate = this.date;
    }

    onClickReset(){
        this.date = this.savedDateState.date;
        this.dateRange.startDate = this.savedDateState.startDate;
        this.dateRange.endDate = this.savedDateState.endDate;
        this.monthDays = this.savedDateState.monthDays;
        this.toMonthDays = this.savedDateState.toMonthDays;
        this.preConfiguredRange = this.savedDateState.preConfiguredRange;
        this.done();
    }

    onClickResetRange() {
        this.done();
    }

    isDateToday(selectedDate) {
        return selectedDate.getDate() == this.today.getDate() && selectedDate.getMonth() == this.today.getMonth() && selectedDate.getFullYear() == this.today.getFullYear();
    }

    isDateSelected(selectedDate) {
        return this.date != null && selectedDate.getDate() == this.date.getDate() && selectedDate.getMonth() == this.date.getMonth() && selectedDate.getFullYear() == this.date.getFullYear();
    }

    isLeftDateSelected(selectedDate) {
        return (this.dateRange.startDate != null && selectedDate.getDate() == this.dateRange.startDate.getDate() && selectedDate.getMonth() == this.dateRange.startDate.getMonth() && selectedDate.getFullYear() == this.dateRange.startDate.getFullYear())
            || (this.dateRange.endDate != null && selectedDate.getDate() == this.dateRange.endDate.getDate() && selectedDate.getMonth() == this.dateRange.endDate.getMonth() && selectedDate.getFullYear() == this.dateRange.endDate.getFullYear());
    }

    isRightDateSelected(selectedDate) {
        return (this.dateRange.startDate != null && selectedDate.getDate() == this.dateRange.startDate.getDate() && selectedDate.getMonth() == this.dateRange.startDate.getMonth() && selectedDate.getFullYear() == this.dateRange.startDate.getFullYear())
            || (this.dateRange.endDate != null && selectedDate.getDate() == this.dateRange.endDate.getDate() && selectedDate.getMonth() == this.dateRange.endDate.getMonth() && selectedDate.getFullYear() == this.dateRange.endDate.getFullYear());
    }

    onPreConfiguredRangeChange(event) {
        var index = parseInt(event);
        switch (index) {
            case -1:
                break;

            case 0:
                var currentDate = new Date();
                var nextMonth = new Date();
                this.leftDate = createDate(currentDate.getFullYear(), currentDate.getMonth(), currentDate.getDate(), 0, 0, 0, 0);
                if (nextMonth.getMonth() == 11) {
                    nextMonth = new Date(nextMonth.getFullYear() + 1, 0, 1);
                } else {
                    nextMonth = new Date(nextMonth.getFullYear(), nextMonth.getMonth() + 1, 1);
                }
                this.rightDate = nextMonth;

                this.generateDaysForDateRange(this.leftDate, this.leftDate);

                break;
            // yesterday
            case 1:
                var currentDate = new Date();
                currentDate.setDate(currentDate.getDate() - 1);
                var nextMonth = new Date(currentDate);
                this.leftDate = createDate(currentDate.getFullYear(), currentDate.getMonth(), currentDate.getDate(), 0, 0, 0, 0);
                if (nextMonth.getMonth() == 11) {
                    nextMonth = new Date(nextMonth.getFullYear() + 1, 0, 1);
                } else {
                    nextMonth = new Date(nextMonth.getFullYear(), nextMonth.getMonth() + 1, 1);
                }
                this.rightDate = nextMonth;
                
                this.generateDaysForDateRange(this.leftDate, this.leftDate);

                break;
            // this month
            case 2:
                var currentDate = new Date();   
                this.leftDate = createDate(currentDate.getFullYear(), currentDate.getMonth(), 1, 0, 0, 0, 0);
                var endDate = new Date(currentDate.getFullYear(), currentDate.getMonth()+1, 0);

                var nextMonth = new Date(currentDate);

                if (nextMonth.getMonth() == 11) {
                    nextMonth = new Date(nextMonth.getFullYear() + 1, 0, 1);
                } else {
                    nextMonth = new Date(nextMonth.getFullYear(), nextMonth.getMonth() + 1, 1);
                }
                this.rightDate = nextMonth;

                this.generateDaysForDateRange(this.leftDate, endDate);
                break;







            case 3:
                this.leftDate = startOfQuarter();
                this.rightDate = endOfQuarter();
                this.generateDaysForDateRange(this.leftDate, this.rightDate);
                break;

            case 4:
                this.leftDate = startOfYear();
                this.rightDate = endOfYear();

                this.generateDaysForDateRange(this.leftDate, this.rightDate);
                break;
  
            case 5:
                this.leftDate = startOfLastWeek();
                this.rightDate = endOfLastWeek();

                this.generateDaysForDateRange(this.leftDate, this.rightDate);
                break;          
                
              case 6:
                this.leftDate = startOfLastMonth();
                this.rightDate = endOfLastMonth();
                this.generateDaysForDateRange(this.leftDate, this.rightDate);
                break;        

            case 7:
                this.leftDate = startOfLastQuarter();
                this.rightDate = endOfLastQuarter();
                this.generateDaysForDateRange(this.leftDate, this.rightDate);
                break;  

            case 8:
                this.leftDate = startOfLastYear();
                this.rightDate = endOfLastYear();
                this.generateDaysForDateRange(this.leftDate, this.rightDate);
                break;                 
        }
    }

    generateDaysForDateRange(startDate: Date, endDate: Date) {
        this.setStartDate(startDate);
        this.setEndDate(endDate);

        var rightDate = this.rightDate;
        if (rightDate.getFullYear() == this.leftDate.getFullYear() && this.rightDate.getMonth() == this.leftDate.getMonth()) {
            rightDate.setMonth(rightDate.getMonth() + 1);
            rightDate.setDate(1);
        }
        this.monthDays = this.generateDays(this.leftDate);
        this.toMonthDays = this.generateDays(rightDate);
    }

    selectEndDate(date: Date) {
        return this.dateRange.endDate != null && date.getFullYear() == this.dateRange.endDate.getFullYear() && date.getMonth() == this.dateRange.endDate.getMonth() && date.getDate() == this.dateRange.endDate.getDate();
    }

    selectStartDate(date: Date) {
        return this.dateRange.startDate != null &&  date.getFullYear() == this.dateRange.startDate.getFullYear() && date.getMonth() == this.dateRange.startDate.getMonth() && date.getDate() == this.dateRange.startDate.getDate();
    }

    hasNoBorder(cbxDateInput: CbxInputDate, cbxDateInput2: CbxInputDate = null) {
        var partialResult = cbxDateInput != null && !cbxDateInput.isActive && cbxDateInput.dateHelper.hasValue() && !this.hasHover && !this.popover;
        if (cbxDateInput2 != null) {
            partialResult = partialResult && cbxDateInput2 != null && !cbxDateInput2.isActive && cbxDateInput2.dateHelper.hasValue()
        }
        return partialResult;
    }

    hasBlueBorder(cbxDateInput: CbxInputDate, cbxDateInput2: CbxInputDate = null) {
        var partialResult = cbxDateInput != null && (cbxDateInput.inputWrapper.hasFocus || this.popover || cbxDateInput.isActive) ;
        if (cbxDateInput2 != null) {
            partialResult = partialResult || cbxDateInput2.inputWrapper.hasFocus || cbxDateInput2.isActive
        }
        return partialResult;
    }

    hasGrayBorder(cbxDateInput: CbxInputDate, cbxDateInput2: CbxInputDate = null) {
        var partialResult = this.hasHover && !cbxDateInput.isActive;
        if (cbxDateInput2 != null) {
            partialResult = partialResult && !cbxDateInput2.isActive
        }
        return partialResult;
    }


    toggleHover(hoverOn: boolean) {
        this.hasHover = hoverOn;
    }

    valueUpdate(event, index) {
        if (this.settings.rangepicker ) {
            if (index == 0) {
                this.dateRange.startDate = event;
            } else if (index == 1) {
                this.dateRange.endDate = event;
            } 
            this.writeValue(this.dateRange);
            this.onChangeCallback(this.dateRange);
        }
        
    }

    compareDates(date1:Date,date2:Date):boolean {
        if ((date1==null)||(date2==null)) return true;
        return date1.getTime() == date2.getTime();
    }

}