import  { pad, formatNames, formatMonth, formatTimeZone, formatDayOfWeek, formatQuarter } from './date-utils';


var formatters = {
    d :function (date, formatLength) {
        return pad(date.getDate(), formatLength);
    },

    E: function (date, formatLength, info) {
        return formatNames(info, "days", formatLength, null)[date.getDay()];
    },
    M: function (date, formatLength, info) {
        return formatMonth(date.getMonth(), formatLength, info, false);
    },

    L: function (date, formatLength, info) {
        return formatMonth(date.getMonth(), formatLength, info, true);
    },

    y : function (date, formatLength) {
        var year = date.getFullYear();
        if (formatLength === 2) {
            year = year % 100;
        }
        return pad(year, formatLength);
    },

    h: function (date, formatLength) {
        var hours = date.getHours() % 12 || 12;
        return pad(hours, formatLength);
    },

    H: function (date, formatLength) {
        return pad(date.getHours(), formatLength);
    },

    k : function (date, formatLength) {
        return pad(date.getHours() || 24, formatLength);
    },

    K : function (date, formatLength) {
        return pad(date.getHours() % 12, formatLength);
    },

    m : function (date, formatLength) {
        return pad(date.getMinutes(), formatLength);
    },

    s : function (date, formatLength) {
        return pad(date.getSeconds(), formatLength);
    },

    S: function (date, formatLength) {
        var milliseconds = date.getMilliseconds();
        var result;
        if (milliseconds !== 0) {
            result = String(date.getMilliseconds() / 1000).split(".")[1].substr(0, formatLength);
        } else {
            result = pad("", formatLength);
        }
        return result;
    },

    a: function (date, formatLength, info) {
        return formatNames(info, "dayPeriods", formatLength, null)[date.getHours() < 12 ? "am" : "pm"];
    },

    z: function (date, formatLength, info) {
        return formatTimeZone(date, info, {
            shortHours: formatLength < 4,
            optionalMinutes: formatLength < 4,
            separator: true,
            localizedName: true
        });
    },

    Z: function (date, formatLength, info) {
        return formatTimeZone(date, info, {
            separator: formatLength > 3,
            localizedName: formatLength === 4,
            zZeroOffset: formatLength === 5
        });
    },

    x : function (date, formatLength, info) {
        return formatTimeZone(date, info, {
            optionalMinutes: formatLength === 1,
            separator: formatLength === 3 || formatLength === 5
        });
    },

    X : function (date, formatLength, info) {
        return formatTimeZone(date, info, {
            optionalMinutes: formatLength === 1,
            separator: formatLength === 3 || formatLength === 5,
            zZeroOffset: true
        });
    },

    G : function (date, formatLength, info) {
        var era = date.getFullYear() >= 0 ? 1 : 0;
        return formatNames(info, "eras", formatLength, null)[era];
    },

    e: formatDayOfWeek,

    c: function (date, formatLength, info) {
        return formatDayOfWeek(date, formatLength, info, true);
    },

    q : function (date, formatLength, info) {
        return formatQuarter(date, formatLength, info, true);
    },
    Q : formatQuarter
};


export default formatters;