
import { lookAhead, round, getNumber, outOfRange, getIndexByName, formatNames, parseMonth, parseTimeZoneOffset, parseDayOfWeek } from './date-utils';



var parsers = {
    d: function (state) {
        lookAhead("d", state);
        var day = getNumber(2, state);

        if (day === null || outOfRange(day, 1, 31)) {
            return true;
        }

        if (state.day === null) {
            state.day = day;
        }
    },
    E: function (state, info) {
        var count = lookAhead("E", state);
        //validate if it matches the day?
        var dayOfWeek = getIndexByName(formatNames(info, "days", count, false, true), state, true);
        if (dayOfWeek === null) {
            return true;
        }
    },
    M: function (state, info) {
        return parseMonth("M", state, info);
    },
    L: function (state, info) {
        return parseMonth("L", state, info);
    },
    y: function (state) {
        var count = lookAhead("y", state);
        var year = getNumber(count === 1 ? undefined : count, state);

        if (year === null) {
            return true;
        }

        if (count === 2) {
            var currentYear = new Date().getFullYear();
            year = (currentYear - currentYear % 100) + year;
            if (year > 2029) {
                year -= 100;
            }
        }

        state.year = year;
    },
    h: function (state) {
        lookAhead("h", state);

        var hours = getNumber(2, state);
        if (hours === 12) {
            hours = 0;
        }

        if (hours === null || outOfRange(hours, 0, 11)) {
            return true;
        }

        state.hours = hours;
    },

    K: function (state) {
        lookAhead("K", state);

        var hours = getNumber(2, state);

        if (hours === null || outOfRange(hours, 0, 11)) {
            return true;
        }

        state.hours = hours;
    },

    a: function (state, info) {
        var count = lookAhead("a", state);
        var periodFormats:any = formatNames(info, "dayPeriods", count, false, true);

        var pmHour = getIndexByName([periodFormats.pm], state, true);
        if (!pmHour && !getIndexByName([periodFormats.am], state, true)) {
            return true;
        }

        state.pmHour = pmHour;
    },

    H: function (state) {
        lookAhead("H", state);
        var hours = getNumber(2, state);
        if (hours === null || outOfRange(hours, 0, 23)) {
            return true;
        }
        state.hours = hours;
    },

    k: function (state) {
        lookAhead("k", state);

        var hours = getNumber(2, state);

        if (hours === null || outOfRange(hours, 1, 24)) {
            return true;
        }

        state.hours = hours === 24 ? 0 : hours;
    },

    m: function (state) {
        lookAhead("m", state);
        var minutes = getNumber(2, state);

        if (minutes === null || outOfRange(minutes, 0, 59)) {
            return true;
        }

        state.minutes = minutes;
    },

    s: function (state) {
        lookAhead("s", state);
        var seconds = getNumber(2, state);
        if (seconds === null || outOfRange(seconds, 0, 59)) {
            return true;
        }
        state.seconds = seconds;
    },

    S: function (state) {
        var count = lookAhead("S", state);
        var match = state.value.substr(state.valueIdx, count);
        var milliseconds = null;

        if (!isNaN(parseInt(match, 10))) {
            milliseconds = parseFloat("0." + match);
            milliseconds = round(milliseconds, 3);
            milliseconds *= 1000;
            state.valueIdx += count;
        }

        if (milliseconds === null || outOfRange(milliseconds, 0, 999)) {
            return true;
        }

        state.milliseconds = milliseconds;
    },

    z: function (state, info) {
        var count = lookAhead("z", state);

        var shortFormat = count < 4;

        var invalid = parseTimeZoneOffset(state, info, {
            shortHours: shortFormat,
            optionalMinutes: shortFormat,
            localizedName: true
        });

        if (invalid) {
            return invalid;
        }
    },

    Z: function (state, info) {
        var count = lookAhead("Z", state);

        var invalid = parseTimeZoneOffset(state, info, {
            noSeparator: count < 4,
            zLiteral: count === 5,
            localizedName: count === 4
        });

        if (invalid) {
            return invalid;
        }
    },

    x: function (state, info) {
        var count = lookAhead("x", state);

        var invalid = parseTimeZoneOffset(state, info, {
            noSeparator: count !== 3 && count !== 5,
            optionalMinutes: count === 1
        });
        if (invalid) {
            return invalid;
        }
    },

    X: function (state, info) {
        var count = lookAhead("X", state);

        var invalid = parseTimeZoneOffset(state, info, {
            noSeparator: count !== 3 && count !== 5,
            optionalMinutes: count === 1,
            zLiteral: true
        });
        if (invalid) {
            return invalid;
        }
    },

    G: function (state, info) {
        var count = lookAhead("G", state);
        var eras = formatNames(info, "eras", count, false, true);
        var era = getIndexByName([eras[0], eras[1]], state, true);

        if (era === null) {
            return true;
        }
    },

    e: function (state, info) {
        return parseDayOfWeek("e", state, info);
    },

    c: function (state, info) {
        return parseDayOfWeek("c", state, info);
    }
}

export default parsers;