import { ControlValueAccessor, NG_VALUE_ACCESSOR } from '@angular/forms';
import { Component, Input, EventEmitter, Output, ContentChildren, QueryList, forwardRef, HostBinding, ChangeDetectorRef } from '@angular/core';
import { TemplateParserComponent } from '../../baseclasses/template-parser.component-base';

export const CBX_RADIO_GROUP_VALUE_ACCESSOR : any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => CbxRadioGroupComponent),
  multi: true,
};

@Component({
  selector: 'cbx-radio-group',
  providers: [CBX_RADIO_GROUP_VALUE_ACCESSOR],
  templateUrl: './cbx-radio-group.component.html',
  styleUrls: ['./cbx-radio-group.component.less'],
  host:{
    '[class.cbx]':'true',
    '[class.cbx-state-disabled]':'disabled'
  }
})
export class CbxRadioGroupComponent extends TemplateParserComponent implements ControlValueAccessor {
  public valArray: string[];

  constructor(private cdRef:ChangeDetectorRef){
    super();
    if (this.isMultiple && this.value instanceof Array){
      this.type = 'checkbox';
      //console.log(this.value);
    }
  }
  
  @Input() isMultiple: boolean = false;
  @Input() required: boolean = false;

  onChange = (any) => {};
  onTouched = (any) => {};

  @Input() label: string;
  @Input() tooltip: string;
  @Input() options: any[];

  @Input() dataValue:string = null;
  @Input() dataText:string = null;
  @Input() dataOrderBy:string = null;

  @Input() strTooltip:string = "";

  @HostBinding('class.disabled')
  @Input()
  disabled: boolean = false;

  @Input() type: 'radio' | 'checkbox' = 'radio';

  @Input() display: 'inline' | 'block' = 'block';

  @Output() readonly cbxChange: EventEmitter<any> = new EventEmitter()

  public value: any;

  public sortOptions(options: any[]) {
    return options;
  }

  writeValue( value : any ) : void {
      this.value = value;
  }

  registerOnChange( fn : any ) : void {
    this.onChange = fn;
  }

  registerOnTouched( fn : any ) : void {
    this.onTouched = fn;
  }

  setDisabledState( isDisabled : boolean ) : void {
    this.disabled = isDisabled;
  }

  ngAfterContentInit(){
    super.ngAfterContentInit();

    if (this.isMultiple){
      this.type = 'checkbox';
    }
    //console.log(this.value);
  }

  ngOnInit(){
    this.checkRequiredInputs();
  }

  checkRequiredInputs(){
    if(!this.options){
      console.error('CbxRadioGroupComponent requires [options] property');
    }else if(!this.label && !this.templateDictionary['label']){
      console.error('CbxRadioGroupComponent requires [label] property or label template');
    }
  }

  public onClick(option){
    event.stopPropagation();

    if (this.isMultiple){
      
      if(!this.value){
        this.value = [];
      }

      let val = this.dataValue? option[this.dataValue]:option;
      if (this.value instanceof Array ){
        let i = this.value.indexOf(val);
        if (i>-1){
          this.value.splice(i,1);
        } else {
          this.value.push(val);
        }
      }
    } else {
      this.value = this.dataValue? option[this.dataValue]:option;      
    }
    //console.log(this.value);
    this.onTouched(this.value);
    this.onChange(this.value);
    this.cbxChange.emit(this.value);
  }


}
