import { Component, OnInit, Input, TemplateRef, ElementRef, ViewChild } from '@angular/core';
import Popper from 'popper.js';


@Component({
  selector: 'cbx-tooltip-container',
  templateUrl: './cbx-tooltip-container.component.html',
  styleUrls: ['./cbx-tooltip-container.component.less'],
  host:{
    '[class.cbx-tooltip-hidden]':'!isShown || tooltipDisabled'
  }
})
export class CbxTooltipContainerComponent implements OnInit {

  constructor(private elemRef:ElementRef) { 
    this.hostElem = this.elemRef.nativeElement;
  }

  @ViewChild('innerContainer') vcInnerContainer:ElementRef;

  ngOnInit() {
  }

  ngAfterViewChecked(){
    if(this.isShown && !this.tooltipDisabled){
      this.recomputeShift();
    }
  }

  recomputeShift() {
    if(!this.vcInnerContainer){
      return;
    }

    let parentWidth = this.hostElem.offsetWidth;
    let parentHeight = this.hostElem.offsetHeight;
    let innerContainer:HTMLElement = this.vcInnerContainer.nativeElement;

    let placementMods = this.placement.split("-");
    let primaryPlacement = placementMods[0];
    let shiftPlacement = placementMods[1];

    if(!shiftPlacement){
      innerContainer.style.top = "";
      innerContainer.style.left = "";
      return;
    }

    if(["top","bottom"].includes(primaryPlacement)){
      innerContainer.style.top = "";
      if(shiftPlacement=="start"){
        innerContainer.style.left = `-${parentWidth/2 - 10}px`;
      }else if(shiftPlacement=="end"){
        innerContainer.style.left = `${parentWidth/2 - 10}px`;
      }
      return;
    }

    if(["left","right"].includes(primaryPlacement)){
      innerContainer.style.left = "";
      if(shiftPlacement=="start"){
        innerContainer.style.top = `-${parentHeight/2 - 10}px`;
      }else if(shiftPlacement=="end"){
        innerContainer.style.top = `${parentHeight/2 - 10}px`;
      }
      return;
    }    

  }

  hostElem:HTMLElement = null;

  @Input() tooltipString:string = null;
  @Input() tooltipTemplate:TemplateRef<any> = null;

  @Input() tooltipClassType:string = null;
  @Input() tooltipFontSize:string = null;

  @Input() isShown:boolean = false;
  @Input() placement:string = "bottom";
  @Input() tooltipDisabled:boolean = false;
}
