import {Directive, ElementRef, Output, EventEmitter, HostListener} from '@angular/core';
import { Subscription } from 'rxjs';
import { GlobalEventsService } from '../services/global-events.service';

@Directive({
    selector: '[clickOutside]'
})
export class CbxClickOutsideDirective {
    constructor(private _elementRef: ElementRef, private globalEvSvc:GlobalEventsService) {
    }

    private subs = new Subscription();

    @Output()
    public clickOutside = new EventEmitter<MouseEvent>();

    ngOnInit(){
        this.subs.add(this.globalEvSvc.FocusMapStream.subscribe(ev => this.onClick(ev)));
    }

    ngOnDestroy(){
        this.subs.unsubscribe();
    }

    public onClick(ev:{click:Event,map:Map<Node, boolean>}): void {
        if (!ev.click.target) {
            return;
        }

        const clickedInside = ev.map.has(this._elementRef.nativeElement);
        if (!clickedInside) {
            this.clickOutside.emit(ev.click as MouseEvent);
        }
    }
}