import { Injectable } from '@angular/core';

/**
 * use this service instead of a polyfill number.prototype.xxx
 * since a polyfill ends up being global and might disrupt the client project
 */
@Injectable()
export class NumberUtilitiesService {

  constructor() { }
  private _blanks:any[] = [null, undefined, ''];
  
  /** check if a number has a value, including 0 */
  hasValue(num:number):boolean{
    return !this._blanks.includes(num);
  }

  /** returns true if num is NOT a whole number */
  hasDecimal(num:number):boolean{
    return (num % 1) > 0;
  }

  getTextValue(num:number):string{
    return !this.hasValue(num)? '':`${num}`;
  }

  trimDecimals(num:number, decimalPlaces:number){
    let decimalFactor = Math.pow(10, decimalPlaces);
    let number = parseInt('' + (num * decimalFactor)) / decimalFactor;
    return number;
  }

  /** https://jsperf.com/checkdecimals */
  countDecimals(num:number){
    if (Math.floor(num) === num) return 0;
    return num.toString().split(".")[1].length || 0;
  }

  validateDecimals(inputType: string, inputVal: number, allowDecimals: boolean, decimalPlaces: number): number{
    if(inputType === "number"){
      let txtValue = this.getTextValue(inputVal);
      let numval:Number = inputVal as Number;

      if(!allowDecimals && txtValue.includes(".")){ 
        return Number(numval.toFixed());

      }else if(allowDecimals && txtValue.includes(".") && !this._blanks.includes(decimalPlaces)){

        let decimaldigits = txtValue.substring(txtValue.indexOf(".")+1);
        if(decimaldigits){
          return Number(numval.toFixed(decimalPlaces));
        }

      } else {
        return parseInt(txtValue);
      }

    } else return inputVal;
  }
  
}
